@@ -1,29 +1,37 @@
 #!/bin/sh
-VERSION=0.9
+VERSION=1.0
 DESTINATION=/tmp/keximdb-dist/$VERSION/
 
+KDESVN=svn://anonsvn.kde.org/home/kde
 CHECKOUT=keximdb-tmp
+
+# From trunk/kdenonbeta
+#BRANCH=trunk/kdenonbeta
+
+# From release branch
+BRANCH=branches/work/keximdb/koffice-1.5
+
 #########
 set -e
-SVNPROG="socksify svn"
+SVN="svn"
 DATE=`date +%Y-%m-%d`
 
 mkdir -p "$DESTINATION"
 mkdir $CHECKOUT
 pushd $CHECKOUT
-  echo "1. Checking out kdenonbeta top level"
-  $SVNPROG co -N svn://anonsvn.kde.org/home/kde/trunk/kdenonbeta/
-  cd kdenonbeta
+  echo "1. Checking out top-level build files"
+  $SVN co -N $KDESVN/$BRANCH keximdb
+  cd keximdb
 
   echo "2. Checking out admin dir"
-  $SVNPROG co svn://anonsvn.kde.org/home/kde/trunk/KDE/kde-common/admin
+  $SVN co $KDESVN/branches/KDE/3.5/kde-common/admin
 
   echo "3. Checking out keximdb dir"
-  $SVNPROG up keximdb
+  $SVN up keximdb
   cd ..
 
   echo "4. Creating build system files"
-  svn2dist --no-i18n kdenonbeta keximdb -v $VERSION -b -g -m
+  svn2dist --no-i18n keximdb keximdb -v $VERSION -b -g -m
 
   echo "5. Cleaning unneeded files"
   #Delete debian dir
