@@ -32,7 +32,9 @@ static int _mdb_attempt_bind(MdbHandle *mdb,
 	MdbColumn *col, unsigned char isnull, int offset, int len);
 static char *mdb_num_to_string(MdbHandle *mdb, int start, int datatype, int prec, int scale);
 static char *mdb_date_to_string(MdbHandle *mdb, int start);
-int mdb_copy_ole(MdbHandle *mdb, char *dest, int start, int size);
+#ifdef MDB_COPY_OLE
+static size_t mdb_copy_ole(MdbHandle *mdb, void *dest, int start, int size);
+#endif
 
 static char date_fmt[64] = "%x %X";
 
@@ -88,7 +90,7 @@ mdb_bind_column_by_name(MdbTableDef *table, gchar *col_name, void *bind_ptr, int
  * 
  * Returns: 0 on success.  1 on failure.
  */
-int mdb_find_pg_row(MdbHandle *mdb, int pg_row, void **buf, int *off, int *len)
+int mdb_find_pg_row(MdbHandle *mdb, int pg_row, void **buf, int *off, size_t *len)
 {
 	unsigned int pg = pg_row >> 8;
 	unsigned int row = pg_row & 0xff;
@@ -102,7 +104,7 @@ int mdb_find_pg_row(MdbHandle *mdb, int pg_row, void **buf, int *off, int *len)
 	return 0;
 }
 
-int mdb_find_row(MdbHandle *mdb, int row, int *start, int *len)
+int mdb_find_row(MdbHandle *mdb, int row, int *start, size_t *len)
 {
 	int rco = mdb->fmt->row_count_offset;
 	int next_start;
@@ -161,20 +163,23 @@ int bit_num = (col_num - 1) % 8;
 }
 /* bool has to be handled specially because it uses the null bit to store its 
 ** value*/
-static int 
+static size_t 
 mdb_xfer_bound_bool(MdbHandle *mdb, MdbColumn *col, int value)
 {
-	
 	col->cur_value_len = value;
 	if (col->bind_ptr) {
-		strcpy(col->bind_ptr,  value ? "0" : "1");
+		strcpy(col->bind_ptr, value ? "0" : "1");
+	}
+	if (col->len_ptr) {
+		*col->len_ptr = 1;
 	}
 
-	return 0;
+	return 1;
 }
-static int mdb_xfer_bound_ole(MdbHandle *mdb, int start, MdbColumn *col, int len)
+static size_t
+mdb_xfer_bound_ole(MdbHandle *mdb, int start, MdbColumn *col, int len)
 {
-	int ret = 0;
+	size_t ret = 0;
 	if (len) {
 		col->cur_value_start = start;
 		col->cur_value_len = len;
@@ -182,16 +187,23 @@ static int mdb_xfer_bound_ole(MdbHandle *mdb, int start, MdbColumn *col, int len
 		col->cur_value_start = 0;
 		col->cur_value_len = 0;
 	}
+#ifdef MDB_COPY_OLE
 	if (col->bind_ptr || col->len_ptr) {
-		
-		memcpy(col->bind_ptr, &mdb->pg_buf[start], MDB_MEMO_OVERHEAD);
+		ret = mdb_copy_ole(mdb, col->bind_ptr, start, len);
 	}
+#else
+	if (col->bind_ptr) {
+		memcpy(col->bind_ptr, mdb->pg_buf + start, MDB_MEMO_OVERHEAD);
+	}
+	ret = MDB_MEMO_OVERHEAD;
+#endif
 	if (col->len_ptr) {
-		*col->len_ptr = MDB_MEMO_OVERHEAD;
+		*col->len_ptr = ret;
 	}
 	return ret;
 }
-static int mdb_xfer_bound_data(MdbHandle *mdb, int start, MdbColumn *col, int len)
+static size_t
+mdb_xfer_bound_data(MdbHandle *mdb, int start, MdbColumn *col, int len)
 {
 int ret;
 	
@@ -235,7 +247,8 @@ int mdb_read_row(MdbTableDef *table, unsigned int row)
 	MdbColumn *col;
 	unsigned int i;
 	int rc;
-	int row_start, row_size;
+	int row_start;
+	size_t row_size;
 	int delflag, lookupflag;
 	MdbField fields[256];
 	int num_fields;
@@ -269,7 +282,7 @@ int mdb_read_row(MdbTableDef *table, unsigned int row)
 #endif 
 
 #if MDB_DEBUG
-	buffer_dump(mdb->pg_buf, row_start, row_start + row_size - 1);
+	buffer_dump(mdb->pg_buf, row_start, row_size);
 #endif
 
 	/* take advantage of mdb_crack_row() to clean up binding */
@@ -437,13 +450,13 @@ int i;
 	return text;
 }
 #endif
-int 
+size_t 
 mdb_ole_read_next(MdbHandle *mdb, MdbColumn *col, void *ole_ptr)
 {
 	guint32 ole_len;
 	void *buf;
 	int row_start;
-	int len;
+	size_t len;
 
 	ole_len = mdb_get_int32(ole_ptr, 0);
 
@@ -457,20 +470,20 @@ mdb_ole_read_next(MdbHandle *mdb, MdbColumn *col, void *ole_ptr)
 			return 0;
 		}
 		if (col->bind_ptr)
-			memcpy(col->bind_ptr, buf + row_start + 4, len - 4);
+			memcpy(col->bind_ptr, (char*)buf + row_start + 4, len - 4);
 		col->cur_blob_pg_row = mdb_get_int32(buf, row_start);
 
 		return len;
 	}
 	return 0;
 }
-int 
+size_t 
 mdb_ole_read(MdbHandle *mdb, MdbColumn *col, void *ole_ptr, int chunk_size)
 {
 	guint32 ole_len;
 	void *buf;
 	int row_start;
-	int len;
+	size_t len;
 
 	ole_len = mdb_get_int32(ole_ptr, 0);
 	mdb_debug(MDB_DEBUG_OLE,"ole len = %d ole flags = %02x",
@@ -481,7 +494,7 @@ mdb_ole_read(MdbHandle *mdb, MdbColumn *col, void *ole_ptr, int chunk_size)
 	if (ole_len & 0x80000000) {
 		/* inline ole field, if we can satisfy it, then do it */
 		len = col->cur_value_len - MDB_MEMO_OVERHEAD;
-		if (chunk_size >= len) {
+		if ((size_t)chunk_size >= len) {
 			if (col->bind_ptr) 
 				memcpy(col->bind_ptr, 
 					&mdb->pg_buf[col->cur_value_start + 
@@ -504,7 +517,7 @@ mdb_ole_read(MdbHandle *mdb, MdbColumn *col, void *ole_ptr, int chunk_size)
 		mdb_debug(MDB_DEBUG_OLE,"start %d len %d", row_start, len);
 
 		if (col->bind_ptr) {
-			memcpy(col->bind_ptr, buf + row_start, len);
+			memcpy(col->bind_ptr, (char*)buf + row_start, len);
 			if (mdb_get_option(MDB_DEBUG_OLE))
 				buffer_dump(col->bind_ptr, 0, 16);
 		}
@@ -517,7 +530,7 @@ mdb_ole_read(MdbHandle *mdb, MdbColumn *col, void *ole_ptr, int chunk_size)
 			return 0;
 		}
 		if (col->bind_ptr) 
-			memcpy(col->bind_ptr, buf + row_start + 4, len - 4);
+			memcpy(col->bind_ptr, (char*)buf + row_start + 4, len - 4);
 		col->cur_blob_pg_row = mdb_get_int32(buf, row_start);
 
 		return len;
@@ -526,11 +539,12 @@ mdb_ole_read(MdbHandle *mdb, MdbColumn *col, void *ole_ptr, int chunk_size)
 		return 0;
 	}
 }
-int mdb_copy_ole(MdbHandle *mdb, char *dest, int start, int size)
+#ifdef MDB_COPY_OLE
+static size_t mdb_copy_ole(MdbHandle *mdb, void *dest, int start, int size)
 {
 	guint32 ole_len;
 	gint32 row_start, pg_row;
-	gint32 len;
+	size_t len;
 	void *buf, *pg_buf = mdb->pg_buf;
 
 	if (size<MDB_MEMO_OVERHEAD) {
@@ -559,7 +573,7 @@ int mdb_copy_ole(MdbHandle *mdb, char *dest, int start, int size)
 			pg_row & 0xff, row_start, len);
 
 		if (dest)
-			memcpy(dest, buf + row_start, len);
+			memcpy(dest, (char*)buf + row_start, len);
 		return len;
 	} else if ((ole_len & 0xff000000) == 0) { 
 		/* multi-page */
@@ -589,11 +603,12 @@ int mdb_copy_ole(MdbHandle *mdb, char *dest, int start, int size)
 		return 0;
 	}
 }
+#endif
 static char *mdb_memo_to_string(MdbHandle *mdb, int start, int size)
 {
 	guint32 memo_len;
 	gint32 row_start, pg_row;
-	gint32 len;
+	size_t len;
 	void *buf, *pg_buf = mdb->pg_buf;
 	char *text = (char *) g_malloc(MDB_BIND_SIZE);
 
@@ -603,7 +618,7 @@ static char *mdb_memo_to_string(MdbHandle *mdb, int start, int size)
 	} 
 
 #if MDB_DEBUG
-	buffer_dump(pg_buf, start, start + MDB_MEMO_OVERHEAD - 1);
+	buffer_dump(pg_buf, start, MDB_MEMO_OVERHEAD);
 #endif
 
 	/* The 32 bit integer at offset 0 is the length of the memo field
@@ -614,7 +629,7 @@ static char *mdb_memo_to_string(MdbHandle *mdb, int start, int size)
 
 	if (memo_len & 0x80000000) {
 		/* inline memo field */
-		mdb_unicode2ascii(mdb, pg_buf + start + MDB_MEMO_OVERHEAD,
+		mdb_unicode2ascii(mdb, (char*)pg_buf + start + MDB_MEMO_OVERHEAD,
 			size - MDB_MEMO_OVERHEAD, text, MDB_BIND_SIZE);
 		return text;
 	} else if (memo_len & 0x40000000) {
@@ -630,9 +645,9 @@ static char *mdb_memo_to_string(MdbHandle *mdb, int start, int size)
 #if MDB_DEBUG
 		printf("row num %d start %d len %d\n",
 			pg_row & 0xff, row_start, len);
-		buffer_dump(buf, row_start, row_start + len - 1);
+		buffer_dump(buf, row_start, len);
 #endif
-		mdb_unicode2ascii(mdb, buf + row_start, len, text, MDB_BIND_SIZE);
+		mdb_unicode2ascii(mdb, (char*)buf + row_start, len, text, MDB_BIND_SIZE);
 		return text;
 	} else if ((memo_len & 0xff000000) == 0) { 
 		/* multi-page memo field */
@@ -657,7 +672,7 @@ static char *mdb_memo_to_string(MdbHandle *mdb, int start, int size)
 			if (tmpoff + len - 4 > memo_len) {
 				break;
 			}
-			memcpy(tmp + tmpoff, buf + row_start + 4, len - 4);
+			memcpy(tmp + tmpoff, (char*)buf + row_start + 4, len - 4);
 			tmpoff += len - 4;
 		} while (( pg_row = mdb_get_int32(buf, row_start) ));
 		if (tmpoff < memo_len) {
@@ -668,7 +683,8 @@ static char *mdb_memo_to_string(MdbHandle *mdb, int start, int size)
 		return text;
 	} else {
 		fprintf(stderr, "Unhandled memo field flags = %02x\n", memo_len >> 24);
-		return "";
+		strcpy(text, "");
+		return text;
 	}
 }
 static char *
@@ -799,7 +815,7 @@ int floor_log10(double f, int is_single)
 
 char *mdb_col_to_string(MdbHandle *mdb, void *buf, int start, int datatype, int size)
 {
-	char *text;
+	char *text = NULL;
 	float tf;
 	double td;
 
@@ -820,13 +836,13 @@ char *mdb_col_to_string(MdbHandle *mdb, void *buf, int start, int datatype, int
 				mdb_get_int32(buf, start));
 		break;
 		case MDB_FLOAT:
-			tf = mdb_get_single(mdb->pg_buf, start);
+			tf = mdb_get_single(buf, start);
 			text = g_strdup_printf("%.*f",
 				FLT_DIG - floor_log10(tf,1) - 1, tf);
 			trim_trailing_zeros(text);
 		break;
 		case MDB_DOUBLE:
-			td = mdb_get_double(mdb->pg_buf, start);
+			td = mdb_get_double(buf, start);
 			text = g_strdup_printf("%.*f",
 				DBL_DIG - floor_log10(td,0) - 1, td);
 			trim_trailing_zeros(text);
@@ -836,7 +852,7 @@ char *mdb_col_to_string(MdbHandle *mdb, void *buf, int start, int datatype, int
 				text = g_strdup("");
 			} else {
 				text = (char *) g_malloc(MDB_BIND_SIZE);
-				mdb_unicode2ascii(mdb, mdb->pg_buf + start,
+				mdb_unicode2ascii(mdb, (char*)buf + start,
 					size, text, MDB_BIND_SIZE);
 			}
 		break;
